==> haystack-0.4.19.tar.gz: lib/src/HDateTimeRange.dart <==
///
/// Copyright (c) 2015, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hdate_time_range;

import 'HVal.dart';
import 'HDate.dart';
import 'HDateTime.dart';
import 'HTimeZone.dart';
import 'ParseError.dart';
import 'io/HZincReader.dart';

/// HDateTimeRange models a starting and ending timestamp
///
/// See [Project Haystack](http://project-haystack.org/doc/Ops#hisRead)
class HDateTimeRange {
  /**
   * Parse from string using the given timezone as context for
   * date based ranges.  The formats are:
   *  - "today"
   *  - "yesterday"
   *  - "{date}"
   *  - "{date},{date}"
   *  - "{dateTime},{dateTime}"
   *  - "{dateTime}"  // anything after given timestamp
   * Throw ParseException is invalid string format.
   */
  factory HDateTimeRange.fromString(String str, HTimeZone tz) {
    // handle keywords
    str = str.trim();
    if (str == "today")     return new HDateTimeRange.dateAndTZ(HDate.today(), tz);
    if (str == "yesterday") return new HDateTimeRange.dateAndTZ(HDate.today().minusDays(1), tz);

    // parse scalars
    int comma = str.indexOf(',');
    HVal start = null, end = null;
    if (comma < 0) {
      start = new HZincReader(str).readVal();
    } else {
      start = new HZincReader(str.substring(0, comma)).readVal();
      end   = new HZincReader(str.substring(comma+1)).readVal();
    }

    // figure out what we parsed for start,end
    if (start is HDate) {
      if (end == null) return new HDateTimeRange.dateAndTZ(start, tz);
      if (end is HDate) return new HDateTimeRange.datesAndTZ(start, end, tz);
    }
    else if (start is HDateTime)
    {
      if (end == null) return new HDateTimeRange(start, new HDateTime.now(tz));
      if (end is HDateTime) return new HDateTimeRange(start, end);
    }

    throw new ParseError("Invalid HDateTimeRange: " + str);
  }

  /** Make for single date within given timezone */
  factory HDateTimeRange.dateAndTZ(HDate date, HTimeZone tz) {
    return new HDateTimeRange.datesAndTZ(date, date, tz);
  }

  /** Make for inclusive dates within given timezone */
  factory HDateTimeRange.datesAndTZ(HDate start, HDate end, HTimeZone tz) {
    return new HDateTimeRange(start.midnight(tz), end.plusDays(1).midnight(tz));
  }

  /** Make from two timestamps */
  factory HDateTimeRange(HDateTime start, HDateTime end) {
    if (start.tz != end.tz) throw new ArgumentError("start.tz and end.tz not equals!");
    return new HDateTimeRange._(start, end);
  }

  /** Make a range which encompasses the current week.
      The week is defined as Sunday thru Saturday. */
  factory HDateTimeRange.thisWeek(HTimeZone tz) {
    HDate today = HDate.today();
    HDate sun = today.minusDays(today.weekday() - DateTime.MONDAY);
    HDate sat = today.plusDays(DateTime.SUNDAY - today.weekday());
    return new HDateTimeRange.datesAndTZ(sun, sat, tz);
  }

  /** Make a range which encompasses the current month. */
  factory HDateTimeRange.thisMonth(HTimeZone tz) {
    HDate today = HDate.today();
    HDate first = new HDate(today.year, today.month, 1);
    HDate last  = new HDate(today.year, today.month, HDate.daysInMonth(today.year, today.month));
    return new HDateTimeRange.datesAndTZ(first, last, tz);
  }

  /** Make a range which encompasses the current year. */
  factory HDateTimeRange.thisYear(HTimeZone tz) {
    HDate today = HDate.today();
    HDate first = new HDate(today.year, 1, 1);
    HDate last  = new HDate(today.year, 12, 31);
    return new HDateTimeRange.datesAndTZ(first, last, tz);
  }

  /** Make a range which encompasses the previous week.
      The week is defined as Sunday thru Saturday. */
  factory HDateTimeRange.lastWeek(HTimeZone tz) {
    HDate today = HDate.today();
    HDate prev = today.minusDays(7);
    HDate sun = prev.minusDays(prev.weekday() - DateTime.MONDAY);
    HDate sat = prev.plusDays(DateTime.SUNDAY - prev.weekday());
    return new HDateTimeRange.datesAndTZ(sun, sat, tz);
  }

  /** Make a range which encompasses the previous month. */
  factory HDateTimeRange.lastMonth(HTimeZone tz) {
    HDate today = HDate.today();
    int year = today.year;
    int month = today.month;

    if (month == 1) {
      year--;
      month = 12;
    } else {
      month--;
    }

    HDate first = new HDate(year, month, 1);
    HDate last  = new HDate(year, month, HDate.daysInMonth(year, month));
    return new HDateTimeRange.datesAndTZ(first, last, tz);
  }

  /** Make a range which encompasses the previous year. */
  factory HDateTimeRange.lastYear(HTimeZone tz) {
    HDate today = HDate.today();
    HDate first = new HDate(today.year-1, 1, 1);
    HDate last  = new HDate(today.year-1, 12, 31);
    return new HDateTimeRange.datesAndTZ(first, last, tz);
  }

  HDateTimeRange._(this.start, this.end);

  /** Inclusive starting timestamp */
  final HDateTime start;

  /** Inclusive ending timestamp */
  final HDateTime end;

  /** Return "start to end" */
  @override
  String toString() => start.toString() + "," + end.toString();
}

==> haystack-0.6.0-dev.0.1.tar.gz: lib/src/HDateTimeRange.dart <==
///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hdate_time_range;

import 'HDate.dart';
import 'HDateTime.dart';
import 'HTimeZone.dart';
import 'HVal.dart';
import 'ParseError.dart';
import 'io/HZincReader.dart';

/// HDateTimeRange models a starting and ending timestamp
///
/// See [Project Haystack](http://project-haystack.org/doc/Ops#hisRead)
class HDateTimeRange {

  /// Parse from string using the given timezone as context for
  /// date based ranges.  The formats are:
  ///  - "today"
  ///  - "yesterday"
  ///  - "{date}"
  ///  - "{date},{date}"
  ///  - "{dateTime},{dateTime}"
  ///  - "{dateTime}"  // anything after given timestamp
  /// Throw ParseException is invalid string format.
  factory HDateTimeRange.fromString(String str, HTimeZone tz) {
    // handle keywords
    str = str.trim();
    if (str == "today")     return new HDateTimeRange.dateAndTZ(HDate.today(), tz);
    if (str == "yesterday") return new HDateTimeRange.dateAndTZ(HDate.today().minusDays(1), tz);

    // parse scalars
    int comma = str.indexOf(',');
    HVal start = null, end = null;
    if (comma < 0) {
      start = new HZincReader(str).readVal();
    } else {
      start = new HZincReader(str.substring(0, comma)).readVal();
      end   = new HZincReader(str.substring(comma+1)).readVal();
    }

    // figure out what we parsed for start,end
    if (start is HDate) {
      if (end == null) return new HDateTimeRange.dateAndTZ(start, tz);
      if (end is HDate) return new HDateTimeRange.datesAndTZ(start, end, tz);
    }
    else if (start is HDateTime)
    {
      if (end == null) return new HDateTimeRange(start, new HDateTime.now(tz));
      if (end is HDateTime) return new HDateTimeRange(start, end);
    }

    throw new ParseError("Invalid HDateTimeRange: " + str);
  }

  /// Make for single date within given timezone
  factory HDateTimeRange.dateAndTZ(HDate date, HTimeZone tz) {
    return new HDateTimeRange.datesAndTZ(date, date, tz);
  }

  /// Make for inclusive dates within given timezone
  factory HDateTimeRange.datesAndTZ(HDate start, HDate end, HTimeZone tz) {
    return new HDateTimeRange(start.midnight(tz), end.plusDays(1).midnight(tz));
  }

  /// Make from two timestamps
  factory HDateTimeRange(HDateTime start, HDateTime end) {
    if (start.tz != end.tz) throw new ArgumentError("start.tz and end.tz not equals!");
    return new HDateTimeRange._(start, end);
  }

  /// Make a range which encompasses the current week.
  ///
  /// The week is defined as Sunday thru Saturday.
  factory HDateTimeRange.thisWeek(HTimeZone tz) {
    HDate today = HDate.today();
    HDate sun = today.minusDays(today.weekday() - DateTime.monday);
    HDate sat = today.plusDays(DateTime.sunday - today.weekday());
    return new HDateTimeRange.datesAndTZ(sun, sat, tz);
  }

  /// Make a range which encompasses the current month.
  factory HDateTimeRange.thisMonth(HTimeZone tz) {
    HDate today = HDate.today();
    HDate first = new HDate(today.year, today.month, 1);
    HDate last  = new HDate(today.year, today.month, HDate.daysInMonth(today.year, today.month));
    return new HDateTimeRange.datesAndTZ(first, last, tz);
  }

  /// Make a range which encompasses the current year.
  factory HDateTimeRange.thisYear(HTimeZone tz) {
    HDate today = HDate.today();
    HDate first = new HDate(today.year, 1, 1);
    HDate last  = new HDate(today.year, 12, 31);
    return new HDateTimeRange.datesAndTZ(first, last, tz);
  }

  /// Make a range which encompasses the previous week.
  ///
  /// The week is defined as Sunday thru Saturday.
  factory HDateTimeRange.lastWeek(HTimeZone tz) {
    HDate today = HDate.today();
    HDate prev = today.minusDays(7);
    HDate sun = prev.minusDays(prev.weekday() - DateTime.monday);
    HDate sat = prev.plusDays(DateTime.sunday - prev.weekday());
    return new HDateTimeRange.datesAndTZ(sun, sat, tz);
  }

  /// Make a range which encompasses the previous month.
  factory HDateTimeRange.lastMonth(HTimeZone tz) {
    HDate today = HDate.today();
    int year = today.year;
    int month = today.month;

    if (month == 1) {
      year--;
      month = 12;
    } else {
      month--;
    }

    HDate first = new HDate(year, month, 1);
    HDate last  = new HDate(year, month, HDate.daysInMonth(year, month));
    return new HDateTimeRange.datesAndTZ(first, last, tz);
  }

  /// Make a range which encompasses the previous year.
  factory HDateTimeRange.lastYear(HTimeZone tz) {
    HDate today = HDate.today();
    HDate first = new HDate(today.year-1, 1, 1);
    HDate last  = new HDate(today.year-1, 12, 31);
    return new HDateTimeRange.datesAndTZ(first, last, tz);
  }

  HDateTimeRange._(this.start, this.end);

  /// Inclusive starting timestamp
  final HDateTime start;

  /// Inclusive ending timestamp
  final HDateTime end;

  /// Return "start to end"
  @override
  String toString() => start.toString() + "," + end.toString();
}
